﻿/* FM-dict.ahk ------ UTF-8 -----------------------------------------------------------------
                 Handle FrameMaker dictionaries (old and new)
   Documentation www.daube.ch/docu/fmaker73.html
   Comment       To be placed in Startup folder of user area
   Reference     Based on SaveBookAsOldVersion 2013-09-04 by Carmen Publishing Inc. (www.frameexpert.com)
   Attention     Code must be UTF-8. With Windows cp no umlauts etc. are allowed in menu items.
   Copyleft      2017 Klaus Daube, Zürich, Switzerland, klaus@daube.ch
   History       2017-07-28 version 1.1 released
                 2017-08-01 version 1.2 released:
                            - Save rather then OK, done with correct function
                            - Dialogue position in the middle of the screen
                 2017-08-04 version 1.3 - not published
                            - List seprator changed from TAB to |
                            - Method from PhiLho (https://autohotkey.com/board/topic/2391-listbox-with-incremental-search/)
                              extended to have search in the ListBox and multiple selection
                            - Optionally allow to work only on new items from the dictionary
                              ATTENTION: due to a severe bug FRMAKER-3201 this can work for the site dictionary!
                  2017-09-28 version 1.4 - not published
                            - Load/save from/to project dictionary
                            - Allow options with separate dialogues
                            - Automatic saves when switching applications
                            - French GUI added
                  2017-09-30 version 1.5 - Import old dicts
                  2017-11-09 version 1.6 - Changes in UI only: introduce group boxes;
                                         - traditional Spelling for German Texts
                  2018-02-15 version 1.7 : new project structure, de-installation programmed
*/ ; // ===========================================================================================

#NoEnv
#SingleInstance Force
#include %A_ScriptDir%\Language\language.ahk                ; support multilingual GUI items

global appVersion                       ; <= config.ini
     , appName                          ; <= config.ini
     , appDicts      := "Not_defined"   ; dictionary dir in AppData
     , compDisplay   := "\-"            ; composita hyphen in display
     , compExternal  := "\-"            ; composita hyphen
     , compInternal  := "-"             ; composita hyphen in internal file
     , compSubst     := "=="            ; intermediate to preserve
     , cfgDir        := "not defined"   ; where config.ini is located
     , cfgFile       := "not defined"
     , CurPrjSrc     := "not defined"
     , DocLang
     , SavPrjCurr    := "Not defined"   ; needs to be global
     , DctTypSrc     := "undefined"
     , fileTime      := "undefined"     ; time stampt for FMUserDictionary file
     , fmVersion     := "undefined"     ; for which FM the utility runs
     , shortList                        ; wordList in "only new words"
     , hyphDisplay   := "-"             ; hyphen in display
     , hyphExternal  := "-"             ; hyphen in old and new files with header
     , hyphInternal  := Chr(2)          ; hyphen in internal file
     , iniFile       := A_ScriptDir . "\FMdict.ini"
     , isDirty       := false           ; has wordlist be changed?
      ; languaged defined in the internal dictionaries - are actually locales: language[_country][.charset] - see
     , languages     := "all|ar_AE|bg_BG|ca_ES|cs_CZ|da_DK|de_CH|de_DE|el_GR|en_CA|en_GB|en_US|es_ES|et_EE|fi_FI|fr_CA|fr_FR|he_IL"
                      . "|hr_HR|hu_HU|it_IT|lt_LT|lv_LV|nb_NO|nl_NL|nn_NO|pl_PL|pt_BR|pt_PT|ro_RO|ru_RU|sk_SK|sl_SI|sv_SE|tr_TR|uk_UA"
     , listSep       := "|"             ; separator of list items
     , logFile       := A_ScriptDir . "\FMdict-log.txt"
     , isLogging     := false
     , isNONE        := "NONE"
     , isUNDEFINED   := "UNDEFINED"
     , haveLoaded    := false           ; true if dictionary lodaded from project
     , langFile      := "undefined"
     , maxWordLen    := 63
     , newHeader     := "<MakerDictionary 3.0>"
     , NL            := "`n"
     , oldHeader     := "<MakerDictionary 2.0>"
     , outDir        := "undefined"
     , PrjDictFile   := "C:\Users\" . A_UserName . "\AppData\Roaming"
     , PrjIndication
     , settingsChanged                  ; UI language, logging, added project
     , sourceFile                       ; mostly the User dictionary
     , srcIsProj     := "Project Dictionary"
     , srcIsSite     := "FMSiteDictionary"
     , srcIsUser     := "FMUserDictionary"
     , targetFile    := ""              ; needs to be global
     , UIlang        := "en"            ; default language
     , userDictionary := "C:\Users\" . A_UserName . "\AppData\LocalLow\Adobe\Linguistics\UserDictionaries"
     , WM_ACTIVATE   := 0x06
     , wordList      := ""
     , AncUIen, AncUIde, AncUIfr, AncLoggingOff, AncLoggingOn, AncDebugOff, AncDebugOn, AncVerboOff, AncVerboOn
     , prjName, prjPath, isGuiAncFunc, isGuiSavFunc

; --- Register watchdog ----------------------------------------------------------------------------------------
  OnMessage(WM_ACTIVATE, "Dlg_GuiActivated")                ; register watchdog function Dlg_GuiActivated

; --- Set up logging / debugging file ---------------------------------------------------------------------------
  If (!FileExist(logFile)) {
      FunLogThis("Log created anew - most likely had been deleted accidentally`n")
  }

; --- Set up the UI language and document language  ------------------------------------------------------------
  lang_init()
  Iniread, inilang, %iniFile%, FMdict, language, %UIlang%
  ; MsgBox, language=%inilang%
  lang_setLanguage(inilang)
  UIlang := _language.lang
  If (UIlang == "de=") {
    DocLang := "de"
  } Else {
    DocLang := "en"
  }

; --- Set up logging -------------------------------------------------------------------------------------------
  Iniread, isLogging, %iniFile%, FMdict, logging, 0
  isLogging=%isLogging%
  FunLogThis(appName .  " [" . appVersion . "] starting =====================================", true)
  Iniread, AncDebugOn, %iniFile%, FMdict, debugging, 0
  Iniread, AncVerboOn, %iniFile%, FMdict, verbose, 0
  If (AncVerboOn)
    AncDebugOn := true
; --- get items from the config.ini (1 level up) --------------------------------------------------
  cfgDir := SubStr(A_ScriptDir, 1, -8)                      ; remove \scripts
  cfgFile := cfgDir . "\config.ini"
  FunGetCfgData()                                           ; appName, appVersion
; --- prepare items for dialogues -----------------------------------------------------------------
  siteDictLocation := FunGetSiteDictLocation()              ; FM takes it from there, FM does not update it
  DctTypSrc      := srcIsUser
  isLanguage     := "all"
  displayDict    := userDictionary
  langFile       := "\" . DctTypSrc . "\" . isLanguage . "\added.txt"
  sourceFile     := userDictionary . langFile
  selWord        := lang("UItxt10")

  Information    := lang("UIinfo", hyphDisplay, compDisplay, hyphDisplay)
  projectList    := FunGetPrjNameList()                     ; read from ini file
  If (projectList == "" ) {
    projectList :=  lang("UItxt00")                         ; E M P T Y
  }
  iniPrj := FunSetPrjIndication()

Dlg_MainGUI:  ; -------------------- Main dialgoue ------------------------------------------------
title := appName . " [" . appVersion . "] — " . lang("UItitle") . " FM-" . fmVersion
Menu, Tray, Icon, %A_ScriptDir%\FMdict.ico

;Gui, Destroy                                               ; avoid multiple use of vars
Gui, New, +OwnDialogs, %title%
Gui, Font, w700, MS sans serif          ; bold
; Gui, Add, Text,         x005 y005 w125 h020 ,                           % lang("UItxt01") ; Dictionary:
Gui, Add, GroupBox,     x005 y000 w710 h075,                            % lang("UItxt01") ; Dictionary type
Gui, Add, Radio,        x010 y020 w120 h020 gDlgSelTypSrc vUserDict checked, % lang("UIrad01")
Gui, Add, Radio,        x140 y020 w080 h020 gDlgSelTypSrc vPrjDict,     % lang("UIrad03")
Gui, Add, Radio,        x440 y020 w120 h020 gDlgSelTypSrc vSiteDict,    % lang("UIrad02")
Gui, Add, DropDownList, x220 y020 w194 sort gDlgSelPrjSrc vCurPrjSrc disabled, %projectList%
; Gui, Add, Button,     x600 y025 w090 h020 gDlgRefresh     ,           Refresh   ; <<<<< use while Dlg_GuiActivated is inactive
Gui, Font, s9 w400, Consolas            ; plain
Gui, Add, Text,         x010 y050 w570 h020         vSourceFile,        %displayDict%…
Gui, Font, w700, MS sans serif          ; bold
; -------------------------------------- Language ---------------------------------------
; Gui, Add, Text,         x005 y060 w090 h020 ,                           % lang("UItxt02") ; Language:
Gui, Add, GroupBox,     x005 y080 w090 h120,                            % lang("UItxt02") ; Language:
Gui, Font
Gui, Add, ListBox,      x010 y100 w075 h100 gDlgSelLangSrc vLanguage,   %languages%
GuiControl, , Language, %languages%
Gui, Font, s9, Consolas                 ; plain
Gui, Add, Text,         x100 y100 w250 h020          vLangFile,         …%langFile%
Gui, Font, w700, MS sans serif          ; bold
Gui, Add, Text,         x440 y100 w280 h020          vPrjIndication,    %PrjIndication%     ; Loaded from project xxx
Gui, Add, Text,         x105 y160 w160 h020 ,                           % lang("UItxt03") ; List displays ...
Gui, Add, Radio,        x280 y160 w160 h020 gDlgSelAllOrNew vgetAll checked, % lang("UIrad04") ; All words
Gui, Add, Radio,        x440 y160 w240 h020 gDlgSelAllOrNew vgetNew,    % lang("UIrad05") ; Only new words
Gui, Font, w700, MS sans serif          ; bold
; -------------------------------------- Dictionary contents ----------------------------
; Gui, Add, Text,         x005 y185 w220 h020 ,                           % lang("UItxt04") ; Dictionary contents
Gui, Add, GroupBox,     x005 y205 w230 h315,                            % lang("UItxt04") ; Dictionary contents
Gui, Font, s10 w400, Consolas           ; code-display
Gui Add, Edit,          x010 y225 w190 h020 gDlgEdtSearchList vToSearch
Gui Add, Button,        x205 y225 W025 h020 gDlgBtnClrSearch,           \\
Gui,Add, ListBox,       x010 y250 w220 h270 vWords multi ,              %wordlist%
; -------------------------------------- Transfer buttons -------------------------------
Gui, Add, Button,       x240 y250 w035 h020 gDlgBtnTransfer,            =>
Gui, Add, Text,         x240 y328 w035 h020 ,                           <=
Gui, Add, Text,         x240 y402 w035 h060 ,                           =>    =>    =>
; -------------------------------------- Edit word list ---------------------------------
Gui, Font, w700, MS sans serif          ; bold
; Gui, Add, Text,         x280 y185 w305 h020 ,                           % lang("UItxt05") ; Selected / new word
Gui, Add, GroupBox,     x280 y205 w435 h150,                            % lang("UItxt05") ; Selected / new word
Gui, Font, s12 w400, Consolas
Gui, Add, Edit,         x285 y225 w425 h045            vEditWord ,      %selWord%     ; was h100
Gui, Font                               ; standrd
Gui, Add, Text,         x285 y275 w375 h055 ,                           %Information%
Gui, Font, w700, MS sans serif          ; bold
Gui, Add, Button,       x285 y325 w125 h020 gDlgBtnReplace,             % lang("UIbtn02")
Gui, Add, Button,       x440 y325 w120 h020 gDlgBtnAdd,                 % lang("UIbtn03")
Gui, Add, Button,       x590 y325 w120 h020 gDlgBtnDelete,              % lang("UIbtn04")
; -------------------------------------- Move to target location -------------------------
Gui, Font, w700, MS sans serif          ; bold
; Gui, Add, Text,         x280 y340 w435 h020 ,                           % lang("UItxt07")
Gui, Add, GroupBox,     x280 y360 w435 h160,                            % lang("UItxt07") ; Move selected words ...
Gui, Add, Radio,        x285 y380 w205 h020 gDlgSelTypOut vUserDctOut,  % lang("UIrad06")
Gui, Add, Radio,        x285 y465 w205 h020 gDlgSelTypOut vProjDctOut,  % lang("UIrad08")
Gui, Add, DropDownList, x300 y490 w190 sort gDlgSelPrjOut vCurPrjOut disabled, %projectList%
Gui, Font                               ; standrd
Gui, Add, Text,         x285 y405 w205 h060 ,                           % lang("UItxt09")

Gui, Add, ListBox,      x500 y380 w065 h135 gDlgSelLangOut vDlgLangOut, %languages%
Gui, Font, w700, MS sans serif          ; bold
Gui, Add, Button,       x590 y410 w120 h060 gDlgBtnMove disabled,       % lang("UIbtn06")
Gui, Add, Text,         x600 y480 w120 h015          disabled,          % lang("UItxt06") ; Done!
; -------------------------------------- information and buttons ------------------------
Gui, Font, w700, MS sans serif          ; bold
Gui, Add, Button,       x005 y525 w220 h020 gDlgBtnSaveAsPrj,           % lang("UIbtn08")
Gui, Add, Button,       x280 y525 w120 h020 gDlgBtnHelp,                % lang("UIbtn09")
Gui, Add, Button,       x440 y525 w120 h020 gDlgBtnAncFunct,            % lang("UIbtn11")
Gui, Add, Button,       x595 y525 w120 h020 gDlgBtnExit,                % lang("UIbtn10")

Gui, Show,                        h550 w720                 ; place in the middle of the screen
  If (iniPrj == isNONE) {
    GuiControl, disabled, PrjIndication
  }
Return

GUIClose(hwnd) { ; --- Name is fixed ----- Target of X button (for all GUIs here) ------------------------------
; the ID (hwnd) of the GUI is automatically passed to the function
; see details in E:\_DDDprojects\FMdictionary-handling\Tests,Comments\MultipleGui.ahk
  DEBUG("  GUIClose           - isGuiAncFunc=%1% isGuiSavFunc=%2% settingsChanged=%3% isDirty=%4%", isGuiAncFunc, isGuiSavFunc, settingsChanged, isDirty)
  If (isGuiAncFunc && settingsChanged) {                    ; closing Ancillary functions?
    MsgBox, % lang("UIopt05")
  }
  If (isGuiSavFunc && settingsChanged) {
    MsgBox, % lang("UIopt18")
  }
  settingsChanged := false                                  ; reset
return 0
}

Dlg_GuiActivated(wParm) {               ; ≥ 1 = activated, 0 = deactivated
; Return ;                                                  <<<<<<<<<<< activate for debugging
; Extension of this function according to https://msdn.microsoft.com/en-us/library/windows/desktop/ms646274
  DEBUG("  Dlg_GuiActivated   - wParm=%1% -- 0: save words, >0: refresh   ", wParm)
  If (wParm) {                                              ; Coming back from other application
      GoSub, DlgRefresh
  } Else {
    FunSaveWordList(wordList)                               ; leaving to other application
  }
}

DlgRefresh:                             ; currently FM updates only the ALL directory files!
; MsgBox, appDicts=%userDictionary%\%DctTypSrc%\all\added.txt
; MsgBox, fileTime=%fileTime%
  FileGettime, newTime, % userDictionary . "\" . DctTypSrc . "\all\added.txt", M
; MsgBox, newTime=%newTime%
  If (newTime != fileTime && Language != "") {              ; something has changed
    langFile    := "\" . DctTypSrc . "\" . Language . "\added.txt"
    GuiControl,, LangFile, %langFile%
    sourceFile  := userDictionary . langFile
    nLines      := FunGetDictContent(sourceFile)            ; fills wordList
    wordList    := FunSortIt(wordList)                      ; Get same order as in FM
    GuiControl,, Words, %wordList%
    getAll := true
    GuiControl, , % lang("UIrad04"), 1                      ; all words
    fileTime := newTime
  DEBUG("  DlgRefresh        e- sourceFile=%1% nLines=%2% ", sourceFile, nLines)
  }
Return

DlgBtnExit:  ; ---------------------------- Cancel -------------------------------------------------------------
  DEBUG("  DlgBtnExit         - isDirty=%1%  ", isDirty)
  If (isDirty) {                                            ; automatically save
    getAll   := true
    FunSaveWordList(wordList)                               ; automatic save
  }
  ExitApp

DlgBtnHelp:  ; ---------------------------- Help for main dialogue ---------------------------------------------
  DEBUG("  DlgBtnHelp         - item1=%1% item2=%2% item3=%3% ", item1, item2, item3)
; Run, http://www.daube.ch/docu/fmaker77.html
Run, %A_ScriptDir%\FMxxx_OpenPdf.exe /A "nameddest=M8.newlink.Utility" "%A_ScriptDir%\FMdict-%DocLang%.pdf"
Return

DlgBtnAncFunct:  ; ------------------------ Enter dialogue Anc. Functions --------------------------------------
  Gui, Submit, noHide                                       ; get the varaible content
  Anc_FunGUI()
Return

DlgSelTypSrc:  ; --------------------------- Select source dictionary/file -------------------------------------
  DEBUG("  DlgSelTypSrc     1 - isDirty=%1% haveLoaded=%2% UserDict=%3% SiteDict=%4% PrjDict=%5% ", isDirty, haveLoaded, UserDict, SiteDict, PrjDict)
  If (isDirty) {                                            ; automatically save
    getAll   := true
    FunSaveWordList(wordList)                               ; automatic save
  }
  Gui, Submit, noHide                                       ; get the varaible content
; MsgBox, DlgSelTypSrc - haveLoaded=%haveLoaded% UserDict=%UserDict% SiteDict=%SiteDict% PrjDict=%PrjDict% `n iniPrj=%iniPrj% CurPrjSrc=%CurPrjSrc%
    If (haveLoaded && != UserDict) {                        ; Loading is into User Dict
      MsgHaveLoaded(iniPrj)                                 ; current content was loaded from ...
      IfMsgBox, Cancel
        Return
    }
  appDicts0 := A_AppData                                    ; ...\AppData\Roaming
  appDicts0 := StrReplace(appDicts0, "Roaming", "LocalLow") . "\Adobe\Linguistics\UserDictionaries\"
  If (UserDict) {
    DctTypSrc   := srcIsUser
    haveLoaded  := false
    appDicts    := appDicts0 . DctTypSrc
    langList    := FunGetActLanguageList(appDicts)          ; list of 'filled' languages
    isLanguage  := "all"
    langFile    := "\" . DctTypSrc . "\" . isLanguage . "\added.txt"
    If (AncVerboOn)               ;
        DEBUG("  DlgSelTypSrc     - UserDict=%1% DctTypSrc=%2% langFile=%3% ", UserDict, DctTypSrc, langFile)
    displayDict := FunFormatFileName(userDictionary)
    GuiControl,, sourceFile, %displayDict%                  ; display in two lines
    GuiControl,, LangFile, %langFile%
    GuiControl,, Language, %langList%                       ; Rebuild for selection
    GuiControl, disabled,  CurPrjSrc                        ; dropDown list
    GuiControl, enabled,  PrjIndication
    If (iniPrj == lang("UItxt12"))                          ; NONE
      GuiControl, disabled,  PrjIndication
    GuiControl, disabled,  % lang("UIbtn12")                ; Refresh
    FileGettime, fileTime, % userDictionary . "\all\added.txt", M
;                             - 19 chars before -
  DEBUG("  DlgSelTypSrc     2 - item1=%1% item2=%2% item3=%3% ", item1, item2, item3)
  } Else If (SiteDict) {
    DctTypSrc   := srcIsSite
    haveLoaded  := false
    appDicts    := appDicts0 . DctTypSrc
    langList    := FunGetActLanguageList(appDicts)          ; list of 'filled' languages
    isLanguage  := "all"
    langFile    := "\" . DctTypSrc . "\" . isLanguage . "\added.txt"
    If (AncVerboOn)               ;
        DEBUG("  DlgSelTypSrc     - SiteDict=%1% DctTypSrc=%2%  langFile=%3% ", SiteDict, DctTypSrc, langFile)
    displayDict := FunFormatFileName(userDictionary)
    GuiControl,, sourceFile, %displayDict%                  ; display in two lines
    GuiControl,, LangFile, %langFile%
    GuiControl,, Language, %langList%                       ; Rebuild for selection
    GuiControl, disabled,  CurPrjSrc                        ; dropDown list
    GuiControl, disabled,  PrjIndication
    GuiControl, disabled,  % lang("UIbtn12")                ; Refresh
    iniPrj := FunSetPrjIndication()
  } Else If (PrjDict) {
    DctTypSrc    := srcIsUser                               ; Project is loaded into user dict
    If (AncVerboOn)               ;
        DEBUG("  DlgSelTypSrc     - PrjDict=%1% DctTypSrc=%2% ", PrjDict, DctTypSrc)
    GuiControl,, LangFile, ""                               ; no language file
    GuiControl, enabled,  CurPrjSrc                         ; dropDown list
    GuiControl,, sourceFile, ""                             ; display in two lines
    GuiControl, disabled,  % lang("UIbtn12")                ; Refresh
;   further actions are in DlgSelPrjSrc
  }
  GuiControl,, Words, % listSep                             ; clear word list
  getAll := true
  GuiControl, , % lang("UIrad04"), 1                        ; All words
  if (isLogging || AncDebugOn)     ;
    FunLogThis("DlgSelTypSrc       e: DctTypSrc = " . DctTypSrc)
Return

DlgSelPrjSrc:  ; ------------------------- Select input project ------------------------------------------------
  Gui, Submit, noHide                                       ; get the variable CurPrjSrc
  Iniread,iniPrj, %iniFile%, FMdict, currentPrj
  if (isLogging)                    ;
    FunLogThis("DlgSelPrjSrc      1 : iniPrj = " . iniPrj " CurPrjSrc = " . CurPrjSrc)
  If (iniPrj == "ERROR") {                                  ; Prj list is empty
    GuiControl, disabled,  CurPrjSrc                        ; dropDown list
    GuiControl, disabled,  PrjIndication                    ; text below drop down
    Return
  }
  If (iniPrj != CurPrjSrc) {
    If (dirty) {
      MsgDictChanged(iniPrj)                                ; laoded from project and modified
      IfMsgBox, Cancel
        Return
    } Else {
      MsgSelProj(iniPrj)                                    ; laoded from project and unchanged
      IfMsgBox, Cancel
        Return
    }
  }
  prjPath := FunGetProjectPath(CurPrjSrc)
  if (isLogging)                    ;
    FunLogThis("DlgSelPrjSrc      2 : iniPrj = " . iniPrj " CurPrjSrc = " . CurPrjSrc . " prjPath = " . prjPath)
  FunLoadProject(CurPrjSrc, prjPath)                        ; also update ini: current project
  haveLoaded  := true
  GuiControl,, UserDict,   1                                ; Switch to user dictionary
  UserDict  := true
  GuiControl, enabled, PrjIndication
  GuiControl, disabled, CurPrjSrc                           ; dropDown list
  GuiControl, , PrjIndication,    % lang("UItxt08") A_Space CurPrjSrc
  appDicts    := appDicts0 . srcIsUser
  langList    := FunGetActLanguageList(appDicts)            ; list of 'filled' languages
; MsgBox, DlgSelPrjSrc - langlist = «%langList%»            ; «|all|de_CH|de_DE|en_GB»
  isLanguage  := "all"
  langFile    := "\FMUserDictionary\" . isLanguage . "\added.txt"
  displayDict := FunFormatFileName(userDictionary)
  GuiControl,, sourceFile, %displayDict%                    ; display in two lines
  GuiControl,, LangFile,   %langFile%
  GuiControl,, Language,   %langList%                       ; Rebuild for selection
Return

DlgSelPrjOut:  ; ------------------------ Select output project ------------------------------------------------
  Gui, Submit, noHide                                       ; get the varaible content
  DEBUG("  DlgSelPrjOut       - item1=%1% item2=%2% item3=%3% ", item1, item2, item3)
  outDir := FunGetProjectPath(CurPrjOut)
Return

DlgSelLangSrc:  ; ------------------------ Select language for source dictionary -------------------------------
  Gui, Submit, noHide                                       ; get the varaible content
  GuiControl, disabled, % lang("UItxt06")                   ; according to label-text
  DEBUG("  DlgSelLangSrc     1- isDirty=%1% DctTypSrc=%2% Language=%3% ", isDirty, DctTypSrc, Language)
  If (isDirty) {                                            ; automatically save
    getAll   := true
    FunSaveWordList(wordList)                               ; automatic save
  }
  If (Language == "")
    Return
  If (DctTypSrc == ProjSrc)
    DctTypSrc := srcIsUser                                  ; has been loaded by project
  langFile    := "\" . DctTypSrc . "\" . Language . "\added.txt"
  GuiControl,, LangFile, %langFile%
  sourceFile  := userDictionary . langFile
  nLines      := FunGetDictContent(sourceFile)              ; fills wordList
  DEBUG("  DlgSelLangSrc     2- sourceFile=%1% nLines=%2% ", sourceFile, nLines)
  wordList    := FunSortIt(wordList)                        ; Get same order as in FM
  GuiControl,, Words, %wordList%
  getAll := true
  GuiControl, , % lang("UIrad04"), 1                        ; all words
  GuiControl, enabled,  % lang("UIbtn12")                   ; Refresh
Return

DlgSelAllOrNew:  ; -------------------------- Get all or only new words from dictionary ------------------------
  Gui, Submit, noHide                                       ; get the varaible content
  DEBUG("  DlgSelAllOrNew     - getAll=%1% sourceFile=%2% ", getAll, sourceFile)
   If (getAll) {
    nLines      := FunGetDictContent(sourceFile)            ; fills wordList
    wordList    := FunSortIt(wordList)                      ; Get same order as in FM
    GuiControl,, Words, %wordList%
    GuiControl, enabled, % lang("UIbtn04")                  ; Delete button
    Return
  } Else {                                                  ; display only new words
    FunSaveWordList(wordList)                               ; automatic save
    savedFile := StrReplace(sourceFile, "added.txt", "added_saved.txt")
    GuiControl, disabled, % lang("UIbtn04")                 ; Delete button
    If (FileExist(savedFile)) {
      shortList := FunGetDiffFile(sourceFile, savedFile)    ; assuming sourceFile is larger
      shortList    := FunSortIt(shortList)                  ; Get same order as in FM
      GuiControl,, Words, %shortList%
      Return
    }                                                       ; no saved word list exist
    wordList := listSep . lang("UItxt00")                   ; EMPTY
    GuiControl,, Words, %wordList%
    Return
  }
Return

DlgBtnTransfer:  ; --------------------------- Transfere word from list to edit field --------------------------
  Gui, Submit, noHide                                       ; get the varaible content
  EditWord := Words
  DEBUG("  DlgBtnTransfer     - EditWord=%1% ", EditWord)
  tPos := InStr(Words, listSep)                             ; multiple items selected?
  If (tPos > 0) {
    EditWord := SubStr(Words, 1, tPos-1)
  }
  GuiControl,, EditWord, %EditWord%
Return

DlgBtnReplace:  ; ------------------------- Replace selected/new word in word-list -----------------------------
  Gui, Submit, noHide                                       ; get the varaible content
  wLen := StrLen(EditWord)
  if (wLen > maxWordLen) {
    MsgWordLimit(EditWord, wLen)
    Return
  }
  toDelete := Words
  If (toDelete != "" ) {
    If (getAll) {                         ;  apend added.txt and in gui list
      wordList   := StrReplace(wordList, listSep . toDelete . listSep, listSep) ; delete old entry
      wordList   := wordList . EditWord . listSep           ; append new entry in Gui list
      wordList   := FunSortIt(wordList)                     ; Get same order as in FM
;     FileAppend, % FunCnvGui2Int(EditWord) . NL, %sourceFile%  ; done elsewhere
      GuiControl,, Words, %wordList%
    } Else {                              ; Only new: append added.txt and short Gui list, added_saved.txt not touched
      shortList   := StrReplace(shortList, listSep . toDelete . listSep, listSep) ; delete old entry
      shortList   := shortList . EditWord . listSep         ; append new entry in Gui list
      shortList   := FunSortIt(shortList)                   ; Get same order as in FM
;     FileAppend, % FunCnvGui2Int(EditWord) . NL, %sourceFile%  ; done elsewhere
      GuiControl,, Words, %shortList%
    }
  }
  GuiControl,, EditWord,                                    ; clear edit-field
  isDirty     := true                                       ; wordlist has changed
  if (isLogging || AncDebugOn)      ;
    FunLogThis("DlgBtnReplace       : " . EditWord)
Return

DlgBtnAdd:  ; ----------------------------- Add word to source dictionary --------------------------------------
  Gui, Submit, noHide                                       ; get the varaible content
  wLen := strLen(EditWord)
  if (wLen > maxWordLen) {
    MsgWordLimit(EditWord, wLen)
    Return
  }
  if (wLen == 0) {
    Return
  }
  DEBUG("  DlgBtnAdd          - getAll=%1% sourceFile=%2% ", getAll, sourceFile)
  If (getAll) {                         ;  apend added.txt and in gui list
    If (wordList == listSep . lang("UItxt00") || wordList == listSep . lang("UItxt11")) { ; |E M P T Y or |Equality
      wordList := listSep
    }
    wordList   := wordList . EditWord . listSep             ; append new entry in Gui list
    wordList   := FunSortIt(wordList)                       ; Get same order as in FM
    FileAppend, % FunCnvGui2Int(EditWord) . NL, %sourceFile%  ; append to added.txt
    GuiControl,, Words, %wordList%
  } Else {                              ; Only new: append added.txt and short Gui list, added_saved.txt not touched
    If (shortList == listSep . lang("UItxt00") || shortList == listSep . lang("UItxt11")) { ; |E M P T Y or |Equality
      shortList := listSep
    }
    shortList   := shortList . EditWord . listSep           ; append new entry in Gui list
    shortList   := FunSortIt(shortList)                     ; Get same order as in FM
    FileAppend, % FunCnvGui2Int(EditWord) . NL, %sourceFile%  ; append to added.txt
    GuiControl,, Words, %shortList%
  }
  GuiControl,, EditWord,                                    ; clear edit-field
  isDirty     := true                                       ; wordlist has changed
  if (isLogging || AncDebugOn)      ;
    FunLogThis("DlgBtnAdd           : " . EditWord)
Return

DlgBtnDelete:  ; -------------------------- Delete word in source dictionary -----------------------------------
  Gui, Submit, noHide                                       ; get the varaible content
  Loop, Parse, Words, %listSep%                             ; does not change sort order
    {
    wordList := StrReplace(wordList, listSep . A_LoopField . listSep, listSep)
    }
  GuiControl,, Words, %wordList%                            ; replace the word list
  isDirty     := true                                       ; wordlist has changed
; FunSaveWordList(wordList)                                 ; automatic save
  if (isLogging || AncDebugOn)      ;
    FunLogThis("DlgBtnDelete        : " . Words)
Return

DlgSelTypOut:  ; -------------------------- Select target dictionary/file --------------------------------------
  Gui, Submit, noHide                                       ; get the varaible content
  GuiControl, disabled, % lang("UItxt06")                   ; according to label-text
  If (Words == "") {
    MsgNothingSelected()
    Guicontrol, , UserDctOut, 0                             ; force user must set
    Guicontrol, , ProjDctOut, 0                             ; force user must set
    Return
  }
  If (UserDctOut) {
    DctTypOut   := srcIsUser
    isLanguage  := "all"
    langFile    := "\" . DctTypOut . "\" . isLanguage . "\added.txt"
    GuiControl,, DlgLangOut,  %listSep%%languages%          ; Rebuild for selection
    GuiControl, disabled, % lang("UIbtn06")                 ; according to button-text
  }
  If (ProjDctOut) {
    DctTypOut    := srcIsProj
    GuiControl, disabled, % lang("UIbtn06")                 ; according to button-text
    GuiControl, enabled,    CurPrjOut                       ; dropDown list
  }
  DEBUG("  DlgSelTypOut      e- DctTypOut=%1% CurPrjOut=%2% langFile=%3% ", DctTypOut, CurPrjOut, langFile)
Return

DlgSelLangOut:  ; ----------------------- Select language for target -------------------------------------------
  Gui, Submit, noHide                                       ; get the varaible content
  If (DctTypOut == srcIsProj) {
    targetFile := outDir . "\" . DlgLangOut . "\added.txt"
  } Else {
    langFileOut := "\" . DctTypOut . "\" . DlgLangOut . "\added.txt"
    targetFile  := userDictionary . langFileOut
  }
    GuiControl, enabled, % lang("UIbtn06")                  ; according to button-text
  DEBUG("  DlgSelLangOut      - DctTypOut=%1% targetFile=%2% ", DctTypOut, targetFile)
Return

DlgBtnMove:  ; ---------------------------- Move words from source to target language --------------------------
; Words        the selection
; wordList     all the words in the dialogue list, starts and ends with listSep
Gui, Submit, noHide                                         ; get the varaible content
  if (targetFile == sourceFile) {
    MsgSameFile(sourceFile)
    Return
  }                                     ; Following always removes all words independently of selection!
  DEBUG("  DlgBtnMove       1 - wordList=%1% ", wordList)
                                        ; --- copy the selected words to the target
  tmpWords := Words
  tmpWords := StrReplace(tmpWords, listSep, NL)             ; create lines
  tmpWords := FunCnvGui2Int(tmpWords)                       ; for the append
  SplitPath, targetFile, , targetDir
  targetDir := targetDir . "\"
  fileAttr := FileExist(targetDir)
  If (fileAttr == "")                   ; directory does not exist
    FileCreateDir, % targetDir
  FileAppend, %tmpWords%`n, % targetDir . "\added.txt", UTF-8

  jWord := 0                            ; --- remove the moved words from the list
  If (SubStr(Words, 1, 1) != listSep) { ; happens after import
    Words := listSep . Words . listSep
  }
  DEBUG("  DlgBtnMove       2 - Words=%1% ", Words)
  Loop, Parse, Words, %listSep%                             ; delete does not change sort order
    {
      jWord += 1
      wordList := StrReplace(wordList, listSep . A_LoopField . listSep, listSep)
      If (AncVerboOn)
        DEBUG("  DlgBtnMove       3 - jWord =%1% wordList=%2% ", jWord, wordList)
    }
                                        ; --- update the UI
  if (isLogging || AncDebugOn)      ;
    FunLogThis("DlgBtnMove          : " . targetDir . " <= " . Words )
  GuiControl,, Words, %wordList%                            ; replace the word list
  Guicontrol, , UserDctOut, 0                               ; force user must set
  Guicontrol, , ProjDctOut, 0                               ; force user must set
  GuiControl, disabled, % lang("UIbtn06")                   ; Move button
  GuiControl, enabled, % lang("UItxt06")                    ; Done
  isDirty := true                                           ; word list has changed
Return

DlgBtnSaveAsPrj:  ; ------------------------ Save project dictionary/ies ---------------------------------------
  Gui, Submit, noHide                                       ; get the varaible content
  Sav_AsPrjGUI()
  if (isLogging || AncDebugOn)      ;
    FunLogThis("DlgBtnSaveAsPrj     : ")
Return

DlgBtnClrSearch:  ; -------------------------- Clear search field ----------------------------------------------
  GuiControl, Choose, Words, 0
	GuiControl, , ToSearch,               ; clear edit field
  if (isLogging || AncDebugOn)      ;
    FunLogThis("DlgBtnClrSearch     : ")
Return

DlgEdtSearchList:  ; ------------------------- Search word (blank terminated) in the list ----------------------
	Gui Submit, NoHide
  If (InStr(ToSearch, " ") == 0)        ; wait for a blank (completion of word)
    Return
  StringTrimRight, ToSearch, ToSearch, 1
	len := StrLen(ToSearch)
	itemNbr := 1
	Loop Parse, wordList, |
	{
		StringLeft part, A_LoopField, len
		If (part == ToSearch) {
			itemNbr := A_Index - 1             ; word list starts with separator
			Break
		}
	}
	GuiControl Choose, Words, %itemNbr%
  if (isLogging || AncDebugOn)      ;
    FunLogThis("DlgEdtSearchList    : itemNbr= " . itemNbr)
Return

; === Functions ================================================================================================
FUN_CTIONS:
Return

Sav_AsPrjGUI() { ; === Save current internal dictionaries as project ===========================================
  Global SavDone1, SavDone2
       , projectList1
       , SavNewPrjName
       , SavNewPrjPath
       , SavPrjCurr
       , SavPrjPath  ; := ""

  isGuiSavFunc := true
  isGuiAncFunc := false
  settingsChanged := false
; MsgBox, CurPrjSrc=%CurPrjSrc%
  projectList1  := FunGetPrjNameList()                      ; read from ini file
  If (projectList1 == "" ) {
    projectList1 :=  lang("UItxt00")                        ; E M P T Y
  }
  If (CurPrjSrc != "") {                                    ; define pre-selection
    projectList1 := StrReplace(projectList1, CurPrjSrc, CurPrjSrc .  listSep)
    If (SubStr(projectList1, 0, 1) == listSep)
      projectList1 := projectList1 . listSep
  }

  title := appName . " [" . appVersion . "] — " . lang("UIsavtitle")
Menu, Tray, Icon, %A_ScriptDir%\FMdict.ico

Gui, New, +OwnDialogs, %title%
Gui, Add, Text,     x005 y005 w440 h040,                                % lang("MsgSaveAsProj")
Gui, Add, Tab3,, % lang("UIsav00")                        ; Save existing|Create new

Gui, Tab, 1 ; --- Save existing
Gui, Add, Text,         x020 y080 w400 h030 ,                           % lang("UIsav01")
Gui, Add, Text,         x020 y120 w060 h020 ,                           % lang("UIopt07")
Gui, Add, DropDownList, x080 y120 w200 sort   gSavPrjSelect vSavPrjCurr,%projectList1%
Gui, Add, Text,         x020 y145 w060 h020 ,                           % lang("UIopt08")
Gui, Font, s9 w400, Consolas            ; plain
Gui, Add, Text,         x080 y145 w350 h040                 vSavPrjPath
Gui, Font, w700, MS sans serif          ; bold
Gui, Add, Button,       x080 y180 w200 h020 gSavBtnAsPrj,               % lang("UIbtn08")
Gui, Add, Text,         x300 y185 w100 h015 vSavDone1 disabled,         % lang("UItxt06")
Gui, Font                               ; standrd
Gui, Add, Text,         x020 y210 w410 h020 ,                           % A_Space  ; to define window size

Gui, Tab, 2 ; --- Create new
Gui, Add, Text,         x020 y080 w400 h030 ,                           % lang("UIsav02")
; Gui, Font                               ; standard
Gui, Add, Text,         x020 y120 w200 h020 ,                           % lang("UIopt06")
Gui, Add, Text,         x020 y145 w060 h020 ,                           % lang("UIopt07")
Gui, Add, Text,         x020 y170 w060 h020 ,                           % lang("UIopt08")
Gui, Add, Text,         x020 y195 w060 h020 ,                           % lang("UIopt09")
Gui, Font, s10 w400, Consolas           ; code-display
Gui, Add, Edit,         x080 y145 w225 h020 vSavNewPrjName,             Client_Whatever
Gui, Add, Edit,         x080 y170 w225 h020 vSavNewPrjPath,             %vSavNewPrjPath%
Gui, Font, w700, MS sans serif          ; bold
Gui, Add, Button,       x320 y170 w115 h020 gSavBtnBrsPrj,              % lang("UIbtn23")
Gui, Add, Button,       x080 y195 w120 h020 gSavBtnSavToNew,            % lang("UIbtn07")
Gui, Add, Text,         x220 y200 w100 h015 vSavDone2 disabled,         % lang("UItxt06")

Gui, Tab    ; ----- on the main surface
Gui, Font, w700, MS sans serif          ; bold
Gui, Add, Button,       x005 y255 w120 h020 gSavBtnHelp,                % lang("UIbtn09")

GUI, Show,                        h280 w450                 ; place in the middle of the screen
Return

; GuiClose: ; -------------------------- (Target of X button) handled in function GUIClose(hwnd) ---------------

SavBtnHelp:  ; --------------------------- Help for dialogue Save As Project -----------------------------------
; Run, http://www.daube.ch/docu/fmaker77.html
; Parameters for Acrobat need to be in this order: /A "page=12" "C:\example.pdf"
Run, %A_ScriptDir%\OpenPdfWithParms.exe /A "nameddest=M8.newlink.SaveAs" "%A_ScriptDir%\FMdict-%DocLang%.pdf"
Return


SavPrjSelect:  ; --------------------------- Select target project for move ------------------------------------
  Gui, Submit, noHide                                       ; get the varaible content
  SavPrjPath := FunGetProjectPath(SavPrjCurr)
  GuiControl, , SavPrjPath, %SavPrjPath%
Return

SavBtnAsPrj:  ; -------------------------- Save current dict ---------------------------------------------------
  Gui, Submit, noHide                                         ; get the varaible content
  UserDict   := "C:\Users\" . A_UserName . "\AppData\LocalLow\Adobe\Linguistics\UserDictionaries\" . DctTypSrc
  if (SavPrjPath == "")
    Return
  FunSaveWordList(wordList)                                 ; upate User dict
  FileCopyDir, %UserDict%, %SavPrjPath%, 1                  ; overwrite.
; However, any files or subfolders inside Dest without counterpart in Source will not be deleted.
  FunDefCurProject(SavPrjCurr, SavPrjPath)                  ; define as current
  GuiControl, enabled, SavDone1
  if (isLogging || AncDebugOn)      ;
    FunLogThis("SavBtnAsPrj         : internal dict saved to " . SavPrjPath)
Return

SavBtnBrsPrj: ; -------------------------- Browse for project folder in Save As Project ------------------------
  FileSelectFolder, SavNewPrjPath, , , Please select a dictionary directory
  if (SavNewPrjPath == "")
    Return
  GuiControl,, SavNewPrjPath, %SavNewPrjPath%
  if (isLogging || AncDebugOn)      ;
    FunLogThis("SavBtnBrsPrj        : ")
Return

SavBtnSavToNew:  ; ----------------------- Save to new Project -------------------------------------------------
  Gui, Submit, noHide                                       ; get the varaible content
  UserDict   := "C:\Users\" . A_UserName . "\AppData\LocalLow\Adobe\Linguistics\UserDictionaries\" . DctTypSrc
  fileAttr := FileExist(SavNewPrjPath)
  If (fileAttr == "")                   ; directory does not exist
    FileCreateDir, % SavNewPrjPath
  FunSaveWordList(wordList)                                 ; upate User dict
  FileCopyDir, %UserDict%, %SavNewPrjPath%, 1               ; overwrite.
; However, any files or subfolders inside Dest without counterpart in Source will not be deleted.
; FunDefCurProject(SavNewPrjName, SavNewPrjPath)            ; define as current
  IniWrite, %SavNewPrjPath%, %iniFile%, Projects, %SavNewPrjName%
  projectList := FunGetPrjNameList()                        ; update list in various locations
  settingsChanged := true
  GuiControl, enabled, SavDone2
  if (isLogging || AncDebugOn)      ;
    FunLogThis("SavBtnSavToNew      : ")
Return

} ; --- End Sav_AsPrjGUI ---------------------------------------------------------------------------------------

Anc_FunGUI() { ; === Dialogue for Ancillary functions ==========================================================
  Global AncDone0, AncDone1, AncDone2, AncDone3, AncDone4, AncDone5
       , AncImpPath
       , AncPrjCurr
       , AncPrjPath
       , AncRemDirs                     ; ON: remove also the project directories
       , projectList2

  isGuiAncFunc := true
  isGuiSavFunc := false
  settingsChanged := false
  appUserDicts    := "%UserProfile%\AppData\LocalLow\Adobe\Linguistics\UserDictionaries\`nFMUserDictionary"
  appSiteDicts    := "%UserProfile%\AppData\LocalLow\Adobe\Linguistics\UserDictionaries\`nFMSiteDictionary"
  Iniread, iniLang, %iniFile%, FMdict, language, en
  Iniread, iniLog,  %iniFile%, FMdict, logging, 0
  Iniread, iniDbg,  %iniFile%, FMdict, debugging, 0
  Iniread, iniVrb,  %iniFile%, FMdict, verbose, 0
  Iniread, iniPrj,  %iniFile%, FMdict, currentPrj, % lang("UItxt12") ; NONE
  Iniread, iniDir,  %iniFile%, FMdict, currentDir, % lang("UItxt13") ; UNDEFINED
  GuiControl, disabled, AncDone1
  projectList2   := FunGetPrjNameList()                     ; read from ini file
  If (projectList2 == "" ) {
    projectList2 := lang("UItxt00")                         ; E M P T Y
  }

  title := appName . " [" . appVersion . "] — " . lang("UIopttitle")
Menu, Tray, Icon, %A_ScriptDir%\FMdict.ico

Gui, New, +OwnDialogs, %title%
Gui, Add, Tab3,, % lang("UIopt00")                          ; Options|New Project|Active dictionaries
Gui, Tab, 1 ; --- Options
Gui, Add, Text,         x020 y050 w090 h020 ,                           % lang("UIopt01")
Gui, Add, Radio,        x100 y050 w100 h020 gAncSetUIlang vAncUIen,       English
Gui, Add, Radio,        x200 y050 w100 h020 gAncSetUIlang vAncUIde,       Deutsch
Gui, Add, Radio,        x300 y050 w100 h020 gAncSetUIlang vAncUIfr,       Français
Gui, Add, Text,         x020 y070 w090 h020 ,                           % lang("UIopt02") ; Logging
Gui, Add, Radio,        x100 y070 w100 h020 gAncSetlog vAncLoggingOn,   % lang("UIopt03")
Gui, Add, Radio,        x200 y070 w100 h020 gAncSetlog vAncLoggingOff,  % lang("UIopt04")
Gui, Add, Button,       x100 y090 w090 h020 gAncBtnLogDisplay  ,        % lang("UIbtn26")
Gui, Add, Button,       x200 y090 w090 h020 gAncBtnLogClear    ,        % lang("UIbtn27")
Gui, Add, Text,         x020 y115 w090 h020 ,                           % lang("UIopt20") ; Debug
Gui, Add, Radio,        x100 y115 w100 h020 gAncSetDbg vAncDebugOn,     % lang("UIopt03")
Gui, Add, Radio,        x200 y115 w100 h020 gAncSetDbg vAncDebugOff,    % lang("UIopt04")
Gui, Add, Text,         x020 y140 w090 h020 ,                           % lang("UIopt21") ; Verbose
Gui, Add, Radio,        x100 y140 w100 h020 gAncSetVrb vAncVerboOn,     % lang("UIopt03")
Gui, Add, Radio,        x200 y140 w100 h020 gAncSetVrb vAncVerboOff,    % lang("UIopt04")

Gui, Tab, 2 ; --- New project
Gui, Font                               ; standard
Gui, Add, Text,         x020 y030 w200 h020 ,                           % lang("UIopt06")
Gui, Add, Text,         x020 y055 w060 h020 ,                           % lang("UIopt07")
Gui, Add, Text,         x020 y080 w060 h020 ,                           % lang("UIopt08")
Gui, Add, Text,         x020 y105 w060 h020 ,                           % lang("UIopt09")
Gui, Font, s10 w400, Consolas           ; code-display
Gui, Add, Edit,          x080 y055 w300 h020 vprjName,                  Client_Whatever
Gui, Add, Edit,          x080 y080 w300 h020 vprjPath,                  %prjPath%
Gui, Font, w700, MS sans serif          ; bold
Gui, Add, Button,       x390 y080 w115 h020 gAncBtnBrsPrj,              % lang("UIbtn23")
Gui, Add, Button,       x080 y105 w120 h020 gAncBtnDefPrj,              % lang("UIbtn24")
Gui, Add, Text,         x220 y110 w100 h015 vAncDone0 disabled,         % lang("UItxt06")

Gui, Tab, 3 ; --- Active dictionaries
Gui, Font                               ; standard
Gui, Add, Text,         x020 y050 w490 h020 ,                           % lang("UIopt13") A_Space "User Dictionary"
Gui, Font, s10 w400, Consolas           ; code-display
Gui, Add, Text,         x020 y065 w490 h040,                            % appUserDicts
Gui, Font                               ; standard
Gui, Add, Text,         x020 y100 w180 h030 ,                           % lang("UIopt14")
Gui, Font, w700, MS sans serif          ; bold
Gui, Add, Button,       x200 y100 w100 h020 gAncBtnRemEmpty2,           % lang("UIbtn25")
Gui, Add, Text,         x320 y105 w100 h015 vAncDone2 disabled,         % lang("UItxt06")
Gui, Font                               ; standard
Gui, Add, Text,         x020 y150 w490 h020 ,                           % lang("UIopt13") A_Space "Site Dictionary"
Gui, Font, s10 w400, Consolas           ; code-display
Gui, Add, Text,         x020 y165 w490 h040,                            % appSiteDicts
Gui, Font                               ; standard
Gui, Add, Text,         x020 y200 w180 h030 ,                           % lang("UIopt14")
Gui, Font, w700, MS sans serif          ; bold
Gui, Add, Button,       x200 y200 w100 h020 gAncBtnRemEmpty3,           % lang("UIbtn25")
Gui, Add, Text,         x320 y205 w100 h015 vAncDone3 disabled,         % lang("UItxt06")

Gui, Tab, 4 ; --- Current project
Gui, Font                               ; standard
Gui, Add, Text,         x020 y050 w180 h020 ,                           % lang("UIopt10")
Gui, Add, Text,         x020 y075 w180 h020 ,                           % lang("UIopt11")
Gui, Add, Text,         x020 y100 w180 h040 ,                           % lang("UIopt12")
Gui, Font, s10 w400, Consolas           ; code-display
Gui, Add, Text,         x200 y050 w300 h020 ,                           % iniPrj
Gui, Add, Text,         x200 y075 w300 h020 ,                           % iniDir
Gui, Font, w700, MS sans serif          ; bold
Gui, Add, Button,       x200 y100 w100 h020 gAncBtnRemEmpty1,           % lang("UIbtn25")
Gui, Add, Text,         x320 y105 w100 h015 vAncDone1 disabled,         % lang("UItxt06")

Gui, Tab, 5 ; --- Delete project
Gui, Add, Text,         x020 y050 w470 h020 ,                           % lang("UIopt16")
Gui, Add, Checkbox,     x020 y070 w470 h020               vAncRemDirs,  % lang("UIopt17")
Gui, Add, Text,         x020 y100 w060 h020 ,                           % lang("UIopt07")
Gui, Add, DropDownList, x080 y100 w200 sort gAncPrjSelect vAncPrjCurr,  %projectList2%
Gui, Add, Text,         x020 y125 w060 h020 ,                           % lang("UIopt08")
Gui, Font, s9 w400, Consolas            ; plain
Gui, Add, Text,         x080 y125 w420 h040                 vAncPrjPath
Gui, Font, w700, MS sans serif          ; bold
Gui, Add, Button,       x080 y160 w200 h020 gAncBtnDelPrj,              % lang("UIbtn28")
Gui, Add, Text,         x300 y165 w100 h015 vAncDone4 disabled,         % lang("UItxt06")

Gui, Tab, 6 ; --- Import dictionary (old, new)
Gui, Font                               ; standard
Gui, Add, Text,         x020 y050 w480 h040 ,                           % lang("UIopt19")
Gui, Add, Text,         x020 y080 w060 h020 ,                           % lang("UIopt08")
Gui, Add, Text,         x020 y105 w060 h020 ,                           % lang("UIopt09")
Gui, Font, s10 w400, Consolas           ; code-display
Gui, Add, Edit,         x080 y080 w300 h020 vAncImpPath,                %AncImpPath%
Gui, Font, w700, MS sans serif          ; bold
Gui, Add, Button,       x390 y080 w115 h020 gAncBtnBrsImp,              % lang("UIbtn23")
Gui, Add, Button,       x080 y105 w120 h020 gAncBtnImport,              % lang("UIbtn29")
Gui, Add, Text,         x220 y110 w100 h015 vAncDone5 disabled,         % lang("UItxt06")

Gui, Tab    ; ----- on the main surface
Gui, Font, w700, MS sans serif          ; bold
Gui, Add, Button,       x005 y225 w120 h020 gAncBtnHelp,                % lang("UIbtn09")

If (iniLang == "en") {
  GuiControl, ,  AncUIen  , 1
} Else If (iniLang == "de") {
  GuiControl, ,  AncUIde  , 1
} Else {
  GuiControl, ,  AncUIfr  , 1
}
If (iniLog == 1) {
  GuiControl, ,  AncLoggingOn  , 1
} Else {
  GuiControl, ,  AncLoggingOff  , 1
}

If (iniDbg == 1) {
  GuiControl, ,  AncDebugOn  , 1
} Else {
  GuiControl, ,  AncDebugOff  , 1
}

If (iniVrb == 1) {
  GuiControl, ,  AncVerboOn  , 1
} Else {
  GuiControl, ,  AncVerboOff  , 1
}

GUI, Show, h250 w520 ,                                                 %title%
Return

; GuiClose: ; -------------------------- (Target of X button) handled in function GUIClose(hwnd) ---------------

AncBtnHelp: ; ---------------------------- Open the corresponding pdf-page for Anc. Functions ------------------
; Run, http://www.daube.ch/docu/fmaker77.html
  Run, %A_ScriptDir%\FMxxx_OpenPdf.exe /A "nameddest=M8.newlink.Ancillary" "%A_ScriptDir%\FMdict-%DocLang%.pdf"
Return

AncSetUIlang:  ; ---------------------- Select language for input ----------------------------------------------
  Gui, Submit, noHide                                       ; get the varaible content
  Iniread,iniLang, %iniFile%, FMdict, language
  If (AncUIen && iniLang != "en") {
    Iniwrite, "en", %iniFile%, FMdict, language
    settingsChanged := true
  }
  If (AncUIde && iniLang != "de") {
    Iniwrite, "de", %iniFile%, FMdict, language
    settingsChanged := true
  }
  If (AncUIfr && iniLang != "fr") {
    Iniwrite, "fr", %iniFile%, FMdict, language
    settingsChanged := true
  }
  DEBUG("  AncSetUIlang       - AncUIen=%1% AncUIde=%2% AncUIfr=%3% ", AncUIen, AncUIde, AncUIfr)
Return

AncSetDbg: ; --------------------------- Define debug settings -------------------------------------------------
  Gui, Submit, noHide                                       ; get the varaible content
  Iniread, iniDbg, %iniFile%, FMdict, debugging
  If (AncDebugOn && iniDbg == 0) {
    Iniwrite, 1, %iniFile%, FMdict, debugging
    settingsChanged := true
  }
  If (AncDebugOff && iniDbg == 1) {
    Iniwrite, 0, %iniFile%, FMdict, debugging
    settingsChanged := true
  }
  DEBUG("  AncSetDbg          - AncDebugOn=%1% AncDebugOff=%2% ", AncDebugOn, AncDebugOff)
Return

AncSetlog:  ; -------------------------- Define Log settings ---------------------------------------------------
  Gui, Submit, noHide                                       ; get the varaible content
  Iniread, iniLog, %iniFile%, FMdict, logging
  If (AncLoggingOn && iniLog == 0) {
    Iniwrite, 1, %iniFile%, FMdict, logging
    settingsChanged := true
  }
  If (AncLoggingOff && iniLog == 1) {
    Iniwrite, 0, %iniFile%, FMdict, logging
    settingsChanged := true
  }
  DEBUG("  AncSetLog          - AncLoggingOn=%1% AncLoggingOff=%2% ", AncLoggingOn, AncLoggingOff)
Return

AncSetVrb: ; --------------------------- Define verbose debug output (in loops) --------------------------------
  Gui, Submit, noHide                                       ; get the varaible content
  Iniread, iniVrb, %iniFile%, FMdict, verbose
  If (AncVerboOn && iniVrb == 0) {
    Iniwrite, 1, %iniFile%, FMdict, verbose
    Iniwrite, 1, %iniFile%, FMdict, logging                ; also set this
    settingsChanged := true
  }
  If (AncVerboOff && iniVrb == 1) {
    Iniwrite, 0, %iniFile%, FMdict, verbose
    settingsChanged := true
  }
  DEBUG("  AncSetVrb          - AncVerboOn=%1% AncVerboOff=%2% ", AncVerboOn, AncVerboOff)
Return

AncBtnLogDisplay:  ; ---------------------- Display log file in standard editor --------------------------------
  Run, %logFile%
  if (isLogging || AncDebugOn)      ;
    FunLogThis("AncBtnLogDisplay    : ")
Return

AncBtnLogClear:  ; ------------------------ Clear log file contents and indicate -------------------------------
  FileDelete, %logFile%
  FunLogThis("Log cleared`n")
  if (isLogging || AncDebugOn)      ;
    FunLogThis("AncBtnLogClear      : ")
Return

AncBtnBrsImp: ; --------------------------- Browse for Import file ---------------------------------------------
  FileSelectFile, AncImpPath, 1, , Please select an old dictionary file (*.dct)
  if (AncImpPath == "")
    Return
  GuiControl, , AncImpPath, %AncImpPath%
  if (isLogging || AncDebugOn)      ;
    FunLogThis("AncBtnBrsImp        : ")
Return

AncBtnImport:  ; -------------------------- Import dictionary file ---------------------------------------------
  Gui, Submit, noHide                                       ; get the varaible content
  If (!FileExist(AncImpPath)) {
    MsgNoFile(fileName)
    Return
  }
  nLines := FunImportOldDict(AncImpPath)
  GuiControl, enabled, AncDone5
  if (isLogging || AncDebugOn)      ;
    FunLogThis("AncBtnImport        : ")
Return

AncBtnBrsPrj: ; --------------------------- Browse for project folder ------------------------------------------
  FileSelectFolder, prjPath, , , Please select a dictionary directory
  if (prjPath == "")
    Return
  GuiControl,, prjPath, %prjPath%
  if (isLogging || AncDebugOn)      ;
    FunLogThis("AncBtnBrsPrj        : ")
Return

AncBtnDefPrj:  ; -------------------------- Create a new project -----------------------------------------------
  Gui, Submit, noHide                                       ; get the varaible content
  SplitPath, prjPath, , , , , drive                         ; including network addresses
  If (drive == "") {
    MsgPathInvalid(prjPath)
    Return
  }
  FunCreateNewProject(prjName, prjPath)
  GuiControl, enabled, AncDone0
  if (isLogging || AncDebugOn)      ;
    FunLogThis("AncBtnDefPrj        : ")
Return

AncBtnDelPrj: ; -------------------------- Delete project ------------------------------------------------------
  Gui, Submit, noHide                                       ; get the varaible content
  Iniread,  iniPrj, %iniFile%, FMdict, currentPrj
  If (iniPrj == AncPrjCurr) {                               ; check for current project ! User must confirm!
    MsgDelCurrent(iniPrj, AncPrjCurr)   ; This action will delete the current project …
    IfMsgBox, Cancel
      Return
  }
  IniDelete, %iniFile%, Projects, %AncPrjCurr%
  IniWrite,  %isNONE%,      %iniFile%, FMdict, currentPrj
  IniWrite,  %isUNDEFINED%, %iniFile%, FMdict, currentDir
  projectList2 := StrReplace(projectList2,  listSep . AncPrjCurr . listSep, listSep ) ; remove item
  GuiControl, , AncPrjCurr, %projectList2%                  ; update list
; MsgBox, AncRemDirs=%AncRemDirs% AncPrjPath=%AncPrjPath%
  If (AncRemDirs) {
    FileRemoveDir, %AncPrjPath%, 1                          ; recurse
  }
  GuiControl, enabled, AncDone4
  GuiControl, , AncRemDirs, 0                               ; uncheck for further use of dialogue
  if (isLogging || AncDebugOn)      ;
    FunLogThis("AncBtnDelPrj        : ")
Return

AncBtnRemEmpty1:  ; -------------------- Remove empty directories from project  --------------------------------
  Iniread, iniDir, %iniFile%, FMdict, currentDir,
  if (isLogging || AncDebugOn)      ;
    FunLogThis("BtnRemoveEmpty      : from " . iniDir)
  If (iniDir == "") {                                       ; globally defined
    MsgBox, 16, FMdict-opt - AncBtnRemEmpty1, Program error: iniDir is empty string
    return
  }
  FunRemoveEmptyDicts(iniDir)
  GuiControl, enabled, AncDone1
  Return

AncBtnRemEmpty2:  ; -------------------- Remove empty directories from FMUserDictionary ------------------------
  appDicts := A_AppData                                     ; ...\AppData\Roaming
  appDicts := StrReplace(appDicts, "Roaming", "LocalLow") . "\Adobe\Linguistics\UserDictionaries\FMUserDictionary"
  FunRemoveEmptyDicts(appDicts)
  GuiControl, enabled, AncDone2
  if (isLogging || AncDebugOn)      ;
    FunLogThis("BtnRemoveEmpty      : from " . appDicts)
Return


AncBtnRemEmpty3:  ; -------------------- Remove empty directories from FMSiteDictionary ------------------------
  appDicts := A_AppData                                     ; ...\AppData\Roaming
  appDicts := StrReplace(appDicts, "Roaming", "LocalLow") . "\Adobe\Linguistics\UserDictionaries\FMSiteDictionary"
  FunRemoveEmptyDicts(appDicts)
  GuiControl, enabled, AncDone3
  if (isLogging || AncDebugOn)      ;
    FunLogThis("BtnRemoveEmpty      : from " . appDicts)
Return

AncPrjSelect:
  Gui, Submit, noHide                                         ; get the varaible content
  AncPrjPath := FunGetProjectPath(AncPrjCurr)
  GuiControl, , AncPrjPath, %AncPrjPath%
  DEBUG("  AncPrjSelect       - AncPrjPath=%1% ", AncPrjPath)
Return

} ; --- End Anc_FunGUI -----------------------------------------------------------------------------------------

FunRemoveEmptyDicts(appDicts) { ; === remove empty dictionary folders ==========================================
  Loop, %appDicts%\*.*, 2                                   ; only folders
  {
; MsgBox, Remove Empty2 - A_LoopFileName=%A_LoopFileName%
    fLen1 := fLen2 := 0                                     ; none of the files may exist
    If (A_LoopFileName == "all")
      Continue                                              ; skip this
    langDir := appDicts . "\" . A_LoopFileName
    FileGetSize, fLen1, %langDir%\added.txt

    If (FileExist(langDir . "\exceptions.txt")) {
      FileGetSize, fLen2, %langDir%\exceptions.txt
    }
; MsgBox, fLen1 fLen2=%fLen1% %fLen2%
    If ((fLen1 == 0 || fLen1 == 3) && (fLen2 == 0 || fLen2 == 3)) {
      FileRemoveDir, %langDir% , 1                          ; recurse
    }
  }
  DEBUG("  FunRemoveEmptyDicts- ")
} ; --- End FunRemoveEmptyDicts

FunCreateNewProject(prjName, prjPath) { ; === Create a new project =============================================
  prjArray      :=  Object()                                ; equivalent of ini section [Projects]

  Iniread, prjList, %iniFile%, Projects                     ; list of existing projects
  nPrj := 0
  If (prjList == "") {                  ; the very first entry
    Iniwrite, %prjPath%, %iniFile%, Projects, %prjName%
    FileCopyDir, %A_ScriptDir%\DictStruct, %prjPath%, 1     ; create target structure (overwriting)
    nPrj := 1
    Return
  }
 	Loop, parse, prjList,`n               ; for ease of handling create an array
	{
  	prjArray.Insert  := A_LoopField                         ; build global array
 	}
  For index, item in prjArray                               ; for all items check whether new location is equal
  {
  	itemArray := StrSplit(item, "=", A_Space)
    projName  := itemArray[1]
    projPath  := itemArray[2]
    nPrj += 1                                               ; count number of projects
    If (projPath == prjPath) {
      MsgPathUsed(projName)                                 ; path already used
      Return
    } Else {                                                ; create new entry and
      Iniwrite, %prjPath%, %iniFile%, Projects, %prjName%
      FileCopyDir, %A_ScriptDir%\DictStruct, %prjPath%, 1   ; create target structure (overwriting)
    }
  }
  DEBUG("  FunCreateNewProject- prjName=%1% prjPath=%2% ", prjName, prjPath)
} ; --- End FunCreateNewProject

FunDefCurProject(prjName, prjPath) { ; === Define project as current ===========================================
  Iniread,  iniPrj, %iniFile%, FMdict, currentPrj
  If (iniPrj != prjName && haveLoaded) {
;   MsgNewCurrent(iniPrj, prjName)      ; The current dictionary was loaded from project %1%. Do you want to save it as another project?
    MsgHaveLoaded(prjName)              ; --- is this more clear?
    IfMsgBox, Cancel
      Return
  }
  Iniwrite, %prjName%, %iniFile%, FMdict, currentPrj       ; indicate current project
  Iniwrite, %prjPath%, %iniFile%, FMdict, currentDir
  if (isLogging || AncDebugOn)      ;
    FunLogThis("FunDefCurProject    : " . prjName . " = " . prjPath)
} ; --- End FunDefCurProject

FunDeleteInDir(directory) { ; === Remove contents of directory =================================================
  DEBUG("  FunDeleteInDir     - directory=%1% ", directory)
  Loop, %directory%\*.*, 2                                  ; 2: retrieve folders only
  {
    FileRemoveDir, %A_LoopFileFullPath%, 1                  ; 1 recurse into
    ErrorCount += ErrorLevel
  }
  return ErrorCount
}

FunLoadProject(prjName, prjPath) { ; === Load proj. dictionary into FMUserDictionary ===========================
  UserDict   := "C:\Users\" . A_UserName . "\AppData\LocalLow\Adobe\Linguistics\UserDictionaries\FMUserDictionary"
  if (isLogging || AncDebugOn)      ;
    FunLogThis("FunLoadProject      : " . prjName . " / " . prjPath)
  FunDeleteInDir(UserDict)                                  ; clear dictionary
  FileCopyDir, %prjPath%, %UserDict%, 1                     ; without overwerite nothing would come in!
;              However, any files or subfolders inside Dest without counterpart in Source will not be deleted.
  FunDefCurProject(prjName, prjPath)
} ; --- End FunLoadProject

FunGetProjectPath(CurPrjSrc) { ; === Get project path from project name in list ================================
  Iniread, prjList, %iniFile%, Projects                     ; list of existing projects
  nPrj := 0
  projectList := ""
 	Loop, parse, prjList,`n               ; for ease of handling create an array
  {
  	itemArray := StrSplit(A_LoopField, "=", A_Space)
    projName  := itemArray[1]
    projPath  := itemArray[2]
    If (projName == CurPrjSrc) {
      Return projPath
    }
  }
  DEBUG("  FunGetProjectPath  - Program error: %1% not found in FMdict.ini [Projects]", CurPrjSrc)
  Return "Error-in-FunGetProjectPath"
} ; --- End FunGetProjectPath

FunGetPrjNameList() { ; === read project names from ini file ===================================================
  Iniread, prjList, %iniFile%, Projects     ; list of existing projects
  nPrj := 0
  projectList := ""
 	Loop, parse, prjList,`n                                   ; for ease of handling create an array
  {
  	itemArray := StrSplit(A_LoopField, "=", A_Space)
    projName  := itemArray[1]
    projPath  := itemArray[2]
    If (nPrj == 0) {
       projectList := projName
    } Else {
      projectList := projectList . listSep . projName
    }
    nPrj += 1                                               ; count number of projects
  }
  DEBUG("  FunGetPrjNameList  - projectList=%1% ", projectList)
  Return projectList
} ; --- End FunGetPrjNameList

FunGetActLanguageList(appDicts) { ; ===  list of actually defined languages ====================================
; MsgBox, Get language list from`n%appDicts%
  langList := ""
  Loop, %appDicts%\*.*, 2                                   ; only folders
  {
    langList := langList . listSep . A_LoopFileName
  }
  DEBUG("  FunGetActLanguageList - langList=%1% ", langList)
  Return langList
} ; --- End FunGetActLanguageList

FunSaveWordList(theList) { ; === Save the (most likely modified) word list =====================================
; Gui, Submit, noHide                                       ; Would overwrite setting in DlgSelAllOrNew
; we are dealing with internal dictionaries only
  if (isLogging || AncDebugOn)      ;
    FunLogThis("FunSaveWordList     : isDirty= " . isDirty)
  If (isDirty) {                        ; has Word list been modified ?
    tmpWords := subStr(theList, 2)                          ; skip first separator in current list
    tmpWords := FunCnvGui2Int(tmpWords)                     ; Gui representation => file repr.
    tmpWords := StrReplace(tmpWords, listSep, NL)           ; create lines from current word list
    SplitPath, sourceFile, , sDir , sExtension, sFileName
    saveFile := sDir . "\" . sFileName . "_saved" . "." . sExtension
    FileMove, %sourceFile%, %saveFile%, 1                   ; overwrite existing
    FileAppend, %tmpWords%, %sourceFile%, UTF-8             ; will be created anew
    isDirty       := false                                  ; theList now 'clean'
  }
  Return
}

FunSortIt(wordList) { ; === get the same order as usual in FM: -xxx, XXX, xxx ==================================
  wordList := StrReplace(wordList, listSep, NL)
  Sort, wordList, C
  wordList := StrReplace(wordList, NL, listSep)
  DEBUG("  FunSortIt         e- WordList=%1% ", wordList)
  Return wordList
} ; --- end FunSortIt

FunFormatFileName(sourceFile) { ; === Break up long file names =================================================
  displayDict := sourceFile                                 ; if short enough
  If (strLen(sourceFile) > 80) {
    bPos := InStr(sourceFile, "\", false, 70)
    If (bPos > 0) {
      displayDict := SubStr(sourceFile, 1, bPos) . " " . SubStr(sourceFile, bPos+1)
    } Else {
      displayDict := SubStr(sourceFile, 1, 80) . " " . SubStr(sourceFile, 81)
    }
  }
  DEBUG("  FunFormatFileName  -`n    sourceFile=%1%`n    displayDict=%2% ", sourceFile, displayDict)
  Return displayDict
} ; --- end FunFormatFileName

FunSetPrjIndication() { ; === Set the projec tientifiction in GUI ==============================================
  Iniread,iniPrj, %iniFile%, FMdict, currentPrj
  If (iniPrj == isNONE) {
    PrjIndication := lang("UItxt08") . A_Space . lang("UItxt12")
  } Else {
    PrjIndication := lang("UItxt08") . A_Space . iniPrj
  }
  GuiControl, , PrjIndication, % PrjIndication
  DEBUG("  FunSetPrjIndication e- iniPrj=%1% ", iniPrj)
  return iniPrj
} ; --- end FunSetPrjIndication

FunGetDictContent(fileName) { ; === Read dictionary file, fill word list =======================================
  wordList  := listSep                                      ; clear list before append
  dictIsOld := false                                        ; new format is default
  internal  := false                                        ; assuming a header
  iLines    := 0
  DEBUG("  FunGetDictContent 1- fileName=%1% ", fileName)
  If (!FileExist(fileName)) {
    MsgNoFile(fileName)
    Return
  }
  FileGetSize, fileSize, %fileName%
  If (fileSize == 0 || fileSize == 3) {                     ; 3: UTF-8 BOM
    wordList := listSep . lang("UItxt00")                   ; E M P T Y
    Return
}

FileEncoding UTF-8                      ; The most essential line before the read loop!
  Loop, READ, %fileName%
  {
    thisLine := A_LoopReadLine
    iLines+= 1
    If (iLines == 1) {                                       ; check dict type
      If (Instr(thisLine, oldHeader)) {
        dictIsOld = true
        Continue                                            ; absorb this line
      }
      If (Instr(thisLine, newHeader)) {
        dictIsOld := false
        internal   := false
        Continue                                            ; absorb this line
      } Else {                                              ; internal dictionary coding
        internal := true
      }
    }
    wordList := wordList . thisLine . listSep
    If (AncVerboOn)
      DEBUG("  FunGetDictContent 2- thisLine=%1%`n  wordList=%2% ", thisLine, wordList)
  }
    If (dictIsOld) {
      wordList := FunCnvFrmRom2Utf(wordList)                ; to presentation
      wordList := FunCnvExt2Gui(wordList)
    } Else {                            ; dict is of new format => to presentation
      If (internal) {                   ; internal dict uses hyphInternal and -
        wordList := FunCnvInt2Gui(wordList)
      } Else {                          ; new dict uses - and \-
        wordList := FunCnvExt2Gui(wordList)
      }
    }
  DEBUG("  FunGetDictContent 3- internal=%1% dictIsOld=%2% iLines=%3% wordList = `n  %4%", internal, dictIsOld, iLines, wordList)
  Return iLines
} ; --- end FunGetDictContent

FunCnvExt2Gui(words) { ; === Convert dct file representation to Gui repr. ======================================
  If (AncVerboOn)
    DEBUG("  FunCnvExt2Gui       - words=%1% ", words)
  words := StrReplace(words, compExternal, compSubst)       ; preserve composita
  words := StrReplace(words, hyphExternal, hyphDisplay )
  words := StrReplace(words, compSubst, compDisplay )
  words := StrReplace(words, NL, listSep)
  Return words
} ; end FunCnvInt2Gui

FunCnvExt2Int(words) { ; === Convert external representation to internal file repr. ============================
  If (AncVerboOn)
    DEBUG("  FunCnvExt2Int       - words=%1% ", words)
  words := StrReplace(words, compExternal, compSubst)       ; preserve composita
  words := StrReplace(words, hyphExternal, hyphInternal)
  words := StrReplace(words, compSubst, compInternal)
  Return words
} ; end FunCnvGui2Int

FunCnvGui2Int(words) { ; === Convert Gui representation to internal file repr. =================================
  If (AncVerboOn)
    DEBUG("  FunCnvGui2Int       - words=%1% ", words)
  words := StrReplace(words, compDisplay, compSubst)        ; preserve composita
  words := StrReplace(words, hyphDisplay, hyphInternal)
  words := StrReplace(words, compSubst, compInternal)
  Return words
} ; end FunCnvGui2Int

FunCnvInt2Gui(words) { ; === Convert internal file representation to Gui repr. =================================
  If (AncVerboOn)
    DEBUG("  FunCnvInt2Gui       - words=%1% ", words)
  words := StrReplace(words, compInternal, compSubst)       ; preserve composita
  words := StrReplace(words, hyphInternal, hyphDisplay)
  words := StrReplace(words, compSubst, compDisplay)
  words := StrReplace(words, NL, listSep)
  Return words
} ; end FunCnvInt2Gui

FunCnvHex2Dec(hexString) { ; === Convert hex string to decimal value ======================================
  hexChars = "0123456789ABCDEF"
  hexLen := StrLen(hexString)                               ; any length possible
  number := 0
  StringUpper, hexUpper, hexString                          ; 8A
  Loop %hexLen% {                                           ; 1    2
    StringMid, hexChar, hexUpper, %A_Index%, 1              ; 8    A
    StringGetPos, decValue, hexChars, %hexChar%             ; 9   11
    number := number*16  + (decValue - 1)                   ; 8*16 + 10 => 138
  }
  If (AncVerboOn)
    DEBUG("  FunCnvHex2Dec       - hexString=%1% number=%2%", hexString, number)
  return number
} ; --- end FunCnvHex2Dec

FunCnvStr2Hex(string) { ; === Convert string to hex string =====================================================
	hex := ""
	n := StrLen(string)
	Loop, %n% {
	  char := SubStr(string, A_Index, 1)
	  code := Asc(char)
	  hChars := Format("{1:02X}", code)
	  hex := hex . hChars
	}
  If (AncVerboOn)
    DEBUG("  FunCnvStr2Hex       - string=%1% hex=%2%", string, hex)
	return hex
} ; end FunCnvStr2Hex

FunCnvFrmRom2Utf(theLine) { ; === convert FrameRoman => Unicode ================================================
; Frame Roman differs from ANSI code only from \x80 (Ä) onwards
;             0    1    2    3    4    5    6    7    8    9    A    B    C    D    E    F      FMcode     index
UTFcode := [ "Ä", "Å", "Ç", "É", "Ñ", "Ö", "Ü", "á", "à", "â", "ä", "ã", "å", "ç", "é", "è" ; 80 ... 8F   1 ...  16
           , "ê", "ë", "í", "ì", "î", "ï", "ñ", "ó", "ò", "ô", "ö", "õ", "ú", "ù", "û", "ü" ; 90 ... 9F  17 ...  32
           , "†", "°", "¢", "£", "§", "•", "¶", "ß", "®", "©", "™", "´", "¨", "¦", "Æ", "Ø" ; A0 ... AF  33 ...  48
           , "×", "±", "ð", "Š", "¥", "μ", "¹", "²", "³", "¼", "½", "ª", "º", "¾", "æ", "ø" ; B0 ... BF  49 ...  64
           , "¿", "¡", "¬", "Ð", "ƒ", "Ý", "ý", "«", "»", "…", "þ", "À", "Ã", "Õ", "Œ", "œ" ; C0 ... CF  65 ...  80
           , "–", "—", "“", "”", "‘", "’", "÷", "Þ", "ÿ", "Ÿ", "⁄", "¤", "‹", "›", "ﬁ", "ﬂ" ; D0 ... DF  81 ...  96
           , "‡", "·", "‚", "„", "‰", "Â", "Ê", "Á", "Ë", "È", "Í", "Î", "Ï", "Ì", "Ó", "Ô" ; E0 ... EF  97 ... 112
           , "š", "Ò", "Ú", "Û", "Ù", "€", "ˆ", "˜", "ˉ", "˘", "Ž", "˚", "¸", "˝", "ž" ]    ; F0 ... FE 113 ... 127

  If (AncVerboOn)
    DEBUG("  FunCnvFrmRom2Utf   - theLine=%1% ", theLine)
  jPos := InStr(theLine, "\x")                              ; first char is 1
  While (jPos > 0) {
    StringMid, FMcode, theLine, jPos+2, 2                   ; e.g. 8A for ä
    codeIndex := FunCnvHex2Dec(FMcode) - 127                ; 138 - 127 = 11
    toSearch :=  "\x" + FMcode
    if (codeIndex > 0 ) {
      toReplace := UtfCode [codeIndex]
    } Else {                        ; ANSI characters (not certain, whether were really used, but who knows
      codeIndex = codeIndex + 127                           ; \x7e => -1 => 126 => ~
      toReplace := Chr(%codeIndex%)                         ; 126 => ~
    }
    theLine := StrReplace(theLine, toSearch, toReplace)
    jPos := InStr(theLine, "\x")
  }
  Return theLine
} ; --- end FunCnvFrmRom2Utf

FunGetDiffFile(newFile, oldFile) { ; === Get difference from old (short) and new (longer) file =================
; - works only on internal dictionaries with no header (header would not be removed)
; - FunGetLineEnd may be different in the two files!
  FileEncoding UTF-8                                        ; Newer files are UniCode
  errTxt := "E R R O R reading file "
  FileRead, newList, %newFile%
  If ErrorLevel {
    DEBUG("  FunGetDiffFile    1- %1% newFile=%2% ", errTxt . "Added.txt", newFile)
    Return listSep . errTxt . "Added.txt"
  }
  endNew  := FunGetLineEnd(newList)
  newList := StrReplace(newList, endNew, listSep)
  FileRead, oldList, %oldFile%
  If ErrorLevel {
    DEBUG("  FunGetDiffFile    2- %1% oldFile=%2% ", errTxt . "Added_saved.txt", oldFile)
    Return listSep . errTxt . "Added_saved.txt"
  }
  endOld  := FunGetLineEnd(oldList)
  oldList := StrReplace(oldList, endOld, listSep)
  Loop, Parse, oldList , %listSep%
    {
    If (A_LoopField == "") {
      Continue                                              ; skip for the terminating separator
    }
    toFind := listSep . A_LoopField . listSep
    If (InStr(newList, toFind)) {                           ; if word from old list is also in new list
      newList := StrReplace(newList, toFind, listSep)       ; delete from new  list
    }                                                       ; remaining must be the only new
  }
  diffList := FunCnvInt2Gui(newList)                        ; file rep => gui rep
  If (diffList == "") {
    diffList := lang("UItxt11")                             ; save == current
  }
  DEBUG("  FunGetDiffFile    e- ")
  Return listSep . diffList
} ; --- end FunGetDiffFile

FunGetLineEnd(lineList) { ; === Determine line-end in read file LF: 0A; CR: 0D =================================
  lineEnd := "00"
  CR       := "`r"                      ; 0D
  CRCR     := "`r`r"
  LF       := "`n"                      ; 0A
  NL       := "`r`n"                    ; 0D 0A (CRLF)
  CRCRLF   := "`r`r`n"
  If InStr(lineList, CRCRLF) > 0 {                          ; CR CR LF
    lineEnd := CRCRLF
  } Else If InStr(lineList, CRCR) > 0 {                     ; CR CR
    lineEnd := CRCR
  } Else If InStr(lineList, NL) > 0 {                       ; CR LF
    lineEnd := NL
  } Else If InStr(lineList, CR) > 0 {                       ; CR
    lineEnd := CR
  } Else If Instr(lineList, LF) > 0 {                       ; LF - internal dicts
    lineEnd := LF
  }
  hexi := FunCnvStr2Hex(lineEnd)
  If (AncVerboOn)
    DEBUG("  FunGetLineEnd      - hexi=%1% ", hexi)
  return lineEnd
} ; --- end FunGetLineEnd

FunGetFMlocation(FMtarget) { ; --- Get FMhome directory of particular or current FM installation -----
; Arguments FMtarget: number of requested FM version (9 or 15 or "9" or "15"=
;                     If empty string, then the current version is searched
; Returns;  Path for FMhome (e.g. H:\Adobe\FrameMaker.15en64\Adobe FrameMaker 2018)
;           Terminating \ is removed for compatibility
; Comment;  64 bit apps write the registry entries in HKEY_LOCAL_MACHINE\SOFTWARE\Adobe\FrameMaker
;           32 bit apps write the registry entries in WOW mode
  If (A_Is64bitOS) {
    SetRegView, 64                      ; view registry as 64 bit app - AHK is 32 bit app
  }
  If (FMtarget != "") {                                     ; search for a specific version of FM
  	RegRead, fmRoot, HKEY_LOCAL_MACHINE, SOFTWARE\Adobe\FrameMaker\%FMtarget%, FMHome
    If (fmRoot == "" ) {
  	  RegRead, fmRoot, HKEY_LOCAL_MACHINE, SOFTWARE\Wow6432Node\Adobe\FrameMaker\%FMtarget%, FMHome
    }
    If (fmRoot == "" ) {                ; FM version can not be found
      Return ""
    } Else {
      If (SubStr(fmRoot, 0) == "\") {
        StringTrimRight, fmRoot, fmRoot, 1 ; remove trailing \
      }
      Return fmRoot
    }
  } Else {                              ; search for the currently active FM version (double click on FM file)
    RegRead, docType, HKEY_CLASSES_ROOT\.fm                 ; Default
    RegRead, command, HKEY_CLASSES_ROOT, %docType%\shell\Open\command  ; Default
    If (command == "" ) {               ; FM version can not be found
      Return ""
    } Else {
      StringTrimRight, fmRoot, command, 21 ; remove trailng \FrameMaker.exe" "%1"
      StringTrimLeft,  fmRoot, fmRoot, 1   ; remove initial "
      Return fmRoot
    }
  }
} ; --- end FunGetFMlocation

FunGetSiteDictLocation() { ; === Get locaton of original site dictionary =======================================
  fmRoot := FunGetFMlocation("")                            ; Get curret FM version
  FileGetVersion, fullVersion , %fmRoot%\FrameMaker.exe     ; Get the current version
  fmVersion  := SubStr(fullVersion, 1, 2)                   ; 12, 13, 14, ...
  FMiniFile := A_AppData . "\Adobe\FrameMaker\" . fmVersion . "\maker.ini"
  Iniread, siteDict,   %FMiniFile%, Files,   SiteDictionary
  If (siteDict != "ERROR") {                                ; we have a local definiton
    if (instr(siteDict, ":") > 0 ) {                        ; absolute path
      siteDictFile := siteDict
    } Else {                                                ; relative path to $HOME
      siteDictFile := fmRoot . siteDict
    }
  } Else {                                                  ; only global definition available
    FMiniLocation := fmRoot . "maker.ini"
    Iniread, siteDict,   %FMiniLocation%, Files,   SiteDictionary
    if (instr(siteDict, ":") > 0 ) {                        ; absolute path
      siteDictFile := siteDict
    } Else {                                                ; relative path to $HOME
      siteDictFile := fmRoot . siteDict
    }
  }
  DEBUG("  FunGetSiteDictLocation - siteDictFile=%1% ", siteDictFile)
  return siteDictFile
} ; --- end FunGetSiteDictLocation

FunImportOldDict(filename) { ; === Read dictionary file, append to language aa_imp ===========================
; Translate form new/old external to internal form
  targetDict := userDictionary . "\FMUserDictionary\aa_imp\"
  If (!FileExist(targetDict)) {         ; directory does not exist
    FileCreateDir, % targetDict
  }
  targetFile := targetDict . "added.txt"

FileEncoding UTF-8                      ; The most essential line before the read loop!
  Loop, READ, %fileName%
  {
    thisLine := A_LoopReadLine
    iLines+= 1
    If (iLines== 1) {                                       ; check dict type
      If (Instr(thisLine, oldHeader)) {
        dictIsOld = true
        Continue                                            ; absorb this line
      }
      If (Instr(thisLine, newHeader)) {
        dictIsOld := false
        internal   := false
        Continue                                            ; absorb this line
      } Else {                                              ; internal dictionary coding
        internal := true
      }
    }

    If (dictIsOld) {                    ; old format (2)
      thisLine := FunCnvFrmRom2Utf(thisLine)
      thisLine := FunCnvExt2Int(thisLine)
    } Else If (!internal) {             ; new format (3)
        thisLine := FunCnvExt2Int(thisLine)
    }                                   ; already internal format
    FileAppend, %thisLine%`n, %targetFile%, UTF-8
  }
  if (isLogging || AncDebugOn)      ;
    FunLogThis("FunImportOldDict    : " . filename)
  Return iLine
} ; --- end FunImportOldDict

FunGetCfgData() {
  IniRead, appName,    %cfgFile%, Project,  appName         ; FMxxxx
  IniRead, appVersion, %cfgFile%, Project,  appVersion      ; 1.2
  IniRead, FMtarget,   %cfgFile%, Project,  FMtarget        ; 12
} ; --- end GetCfgData


FunLogThis(text, isNL = 0) { ; ----- log text --------------------------------------------------------------------
  newLine := ""
  If (isNL) {
    newLine := "`n"
  }
	FormatTime, now, , yyyy-MM-dd HH:mm:ss
	If (AncDebugOn) {                     ; avoid to different entries in this case
	  now := "Debug -  "
	}
	FileAppend %newLine%%now% %text%`n, %logFile%, UTF-8
} ; end FunLogThis

DEBUG(text, p1 = "", p2 = "", p3= "", p4="", p5 = "") { ; ----- log text with at most 5 paramters --------------
  If (AncDebugOn) {
  	text := StrReplace( text, "%1%" , p1)
  	text := StrReplace( text, "%2%" , p2)
  	text := StrReplace( text, "%3%" , p3)
  	text := StrReplace( text, "%4%" , p4)
  	text := StrReplace( text, "%5%" , p5)
  	FileAppend Debug - %text%`n, %logFile%, UTF-8
 }
} ; end DEBUG

; === Messages =================================================================================================

MsgDelCurrent(iniPrj, AncPrjCurr) {
  If (AncVerboOn)
    DEBUG("  MsgDelCurrent      - Msg=%1% ", lang("MsgDelCurrent", iniPrj, AncPrjCurr))
  MsgBox, % 1+32, FMdict — MsgDelCurrent, % lang("MsgDelCurrent", iniPrj, AncPrjCurr)
}

MsgDictChanged(iniPrj) {
  If (AncVerboOn)
    DEBUG("  MsgDictChanged     - Msg=%1% ", lang("MsgDictChanged", iniPrj))
  MsgBox, % 1+32, FMdict — MsgDictChanged, % lang("MsgDictChanged", iniPrj)
}

MsgHaveLoaded(prjName) {
  If (AncVerboOn)
    DEBUG("  MsgHaveLoaded      - Msg=%1% ", lang("MsgHaveLoaded", prjName))
  MsgBox, % 1+32, FMdict — MsgHaveLoaded, % lang("MsgHaveLoaded", prjName)
}

MsgNewCurrent(iniPrj, prjName) {
  If (AncVerboOn)
    DEBUG("  MsgNewCurrent      - Msg=%1% ", lang("MsgNewCurrent", iniPrj, prjName))
  MsgBox, % 1+32, FMdict — MsgNewCurrent, % lang("MsgNewCurrent", iniPrj, prjName)
}

MsgNoFile(fileName) {
  If (AncVerboOn)
    DEBUG("  MsgNoFile          - Msg=%1% ", lang("MsgNoFile", fileName))
  MsgBox, 48, FMdict — MsgNoFile,      % lang("MsgNoFile", fileName)
}

MsgPathInvalid(pathName) {
  If (AncVerboOn)
    DEBUG("  MsgPathInvalid     - Msg=%1% ", lang("MsgPathInvalid", pathName))
  MsgBox, 48, FMdict — MsgPathInvalid,  % lang("MsgPathInvalid", pathName)
}

MsgNonTxtFile(fileName) {
  If (AncVerboOn)
    DEBUG("  MsgNonTxtFile      - Msg=%1% ", lang("MsgNonTxtFile", fileName))
  MsgBox, 48, FMdict — MsgNonTxtFile,  % lang("MsgNonTxtFile", fileName)
}

MsgNothingSelected() {
  If (AncVerboOn)
    DEBUG("  MsgNothingSelected - Msg=%1% ", lang("MsgNothingSelected"))
  MsgBox, 48, FMdict — MsgNothingSelected,  % lang("MsgNothingSelected")
}

MsgPathUsed(projName) {
  If (AncVerboOn)
    DEBUG("  MsgPathUsed        - Msg=%1% ", lang("MsgPathUsed", projName))
  MsgBox, 48, FMdict-opt — MsgPathUsed,  % lang("MsgPathUsed", projName)
}

MsgSameFile(fileName) {
  If (AncVerboOn)
    DEBUG("  MsgSameFile        - Msg=%1% ", lang("MsgSameFile", fileName))
  MsgBox, 48, FMdict — MsgSameFile,    % lang("MsgSameFile", fileName)
}

MsgSelProj(iniPrj) {
  If (AncVerboOn)
    DEBUG("  MsgSelProj         - Msg=%1% ", lang("MsgSelProj", iniPrj))
  MsgBox, % 1+32, FMdict — MsgSelProj, % lang("MsgSelProj", iniPrj)
}

MsgWordLimit(word, wLen) {
  If (AncVerboOn)
    DEBUG("  MsgWordLimit       - Msg=%1% ", lang("MsgWordLimit", word, wLen, maxWordLen))
  MsgBox, 48, FMdict — MsgWordLimit,   % lang("MsgWordLimit", word, wLen, maxWordLen)
}
; Currently not used: --------------------------------------
MsgTest(value) {
  If (AncVerboOn)
    DEBUG("  MsgTest      - Msg=%1% ", lang("MsgTest", value))
}

MsgCancel() {
  MsgBox, % 1+32, FMdict — MsgCancel,  % lang("MsgCancel")
}
MsgNoOldFile(fileName) {
  MsgBox, 48, FMdict — MsgNoOldFile,   % lang("MsgNoOldFile", fileName)
}
MsgNothingChanged() {
  MsgBox, 48, FMdict — MsgNothingChanged, % lang("MsgNothingChanged")
}
MsgNothingNew(fileName) {
  MsgBox, 48, FMdict — MsgNothingNew,  % lang("MsgNothingNew", fileName)
}
MsgNotSaved() {
  MsgBox, 48, FMdict — MsgNotSaved,    % lang("MsgNotSaved")
}
